# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class CreditorsService(base_service.BaseService):
    """Service class that provides access to the creditors
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.Creditor
    RESOURCE_NAME = 'creditors'


    def create(self,params=None, headers=None):
        """Create a creditor.

        Creates a new creditor.

        Args:
              params (dict, optional): Request body.

        Returns:
              Creditor
        """
        path = '/creditors'
        
        if params is not None:
            params = {self._envelope_key(): params}

        try:
          response = self._perform_request('POST', path, params, headers,
                                            retry_failures=True)
        except errors.IdempotentCreationConflictError as err:
          if self.raise_on_idempotency_conflict:
            raise err
          return self.get(identity=err.conflicting_resource_id,
                          params=params,
                          headers=headers)
        return self._resource_for(response)
  

    def list(self,params=None, headers=None):
        """List creditors.

        Returns a [cursor-paginated](#api-usage-cursor-pagination) list of your
        creditors.

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of Creditor instances
        """
        path = '/creditors'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  

    def get(self,identity,params=None, headers=None):
        """Get a single creditor.

        Retrieves the details of an existing creditor.

        Args:
              identity (string): Unique identifier, beginning with "CR".
              params (dict, optional): Query string parameters.

        Returns:
              Creditor
        """
        path = self._sub_url_params('/creditors/:identity', {
          
            'identity': identity,
          })
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def update(self,identity,params=None, headers=None):
        """Update a creditor.

        Updates a creditor object. Supports all of the fields supported when
        creating a creditor.

        Args:
              identity (string): Unique identifier, beginning with "CR".
              params (dict, optional): Request body.

        Returns:
              Creditor
        """
        path = self._sub_url_params('/creditors/:identity', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {self._envelope_key(): params}

        response = self._perform_request('PUT', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def apply_scheme_identifier(self,identity,params=None, headers=None):
        """Apply a scheme identifier.

        Applies a [scheme identifier](#core-endpoints-scheme-identifiers) to a
        creditor.
        
        If the scheme identifier has a `pending` status, it will be applied
        asynchronously
        once it becomes `active`.
        
        If the creditor already has a scheme identifier for the scheme, it will
        be replaced,
        and any mandates attached to it transferred asynchronously. On Bacs and
        SEPA, if
        payments were about to be submitted, they will be delayed. To minimise
        this delay, we
        recommend that you apply the new scheme identifier after the daily
        payment submission
        time (4 PM Europe/London time).
        

        Args:
              identity (string): Unique identifier, beginning with "CR".
              params (dict, optional): Request body.

        Returns:
              Creditor
        """
        path = self._sub_url_params('/creditors/:identity/actions/apply_scheme_identifier', {
          
            'identity': identity,
          })
        
        if params is not None:
            params = {'data': params}
        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class SchemeIdentifiersService(base_service.BaseService):
    """Service class that provides access to the scheme_identifiers
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.SchemeIdentifier
    RESOURCE_NAME = 'scheme_identifiers'


    def create(self,params=None, headers=None):
        """Create a scheme identifier.

        Creates a new scheme identifier. The scheme identifier must be
        [applied to a creditor](#creditors-apply-a-scheme-identifier) before
        payments are taken
        using it. The scheme identifier must also have the `status` of active
        before it can be
        used. On Bacs, this will take 5 working days. On other schemes, this
        happens instantly.
        
        #### Scheme identifier name validations
        
        The `name` field of a scheme identifier can contain alphanumeric
        characters, spaces and
        special characters.
        
        Its maximum length and the special characters it supports depend on the
        scheme:
        
        | __scheme__        | __maximum length__ | __special characters
        allowed__                      |
        | :---------------- | :----------------- |
        :-------------------------------------------------- |
        | `bacs`            | 18 characters      | `/` `.` `&` `-`             
                               |
        | `sepa_core`       | 70 characters      | `/` `?` `:` `(` `)` `.` `,`
        `+` `&` `<` `>` `'` `"` |
        | `ach`             | 16 characters      | `/` `?` `:` `(` `)` `.` `,`
        `'` `+` `-`             |
        | `faster_payments` | 18 characters      | `/` `?` `:` `(` `)` `.` `,`
        `'` `+` `-`             |
        
        The validation error that gets returned for an invalid name will
        contain a suggested name
        in the metadata that is guaranteed to pass name validations.
        
        You should ensure that the name you set matches the legal name or the
        trading name of
        the creditor, otherwise, there is an increased risk of chargeback.
        

        Args:
              params (dict, optional): Request body.

        Returns:
              SchemeIdentifier
        """
        path = '/scheme_identifiers'
        
        if params is not None:
            params = {self._envelope_key(): params}

        try:
          response = self._perform_request('POST', path, params, headers,
                                            retry_failures=True)
        except errors.IdempotentCreationConflictError as err:
          if self.raise_on_idempotency_conflict:
            raise err
          return self.get(identity=err.conflicting_resource_id,
                          params=params,
                          headers=headers)
        return self._resource_for(response)
  

    def list(self,params=None, headers=None):
        """List scheme identifiers.

        Returns a [cursor-paginated](#api-usage-cursor-pagination) list of your
        scheme identifiers.

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of SchemeIdentifier instances
        """
        path = '/scheme_identifiers'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  

    def get(self,identity,params=None, headers=None):
        """Get a single scheme identifier.

        Retrieves the details of an existing scheme identifier.

        Args:
              identity (string): Unique identifier, usually beginning with "SU".
              params (dict, optional): Query string parameters.

        Returns:
              SchemeIdentifier
        """
        path = self._sub_url_params('/scheme_identifiers/:identity', {
          
            'identity': identity,
          })
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class VerificationDetail(object):
    """A thin wrapper around a verification_detail, providing easy access to its
    attributes.

    Example:
      verification_detail = client.verification_details.get()
      verification_detail.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def address_line1(self):
        return self.attributes.get('address_line1')
  

    @property
    def address_line2(self):
        return self.attributes.get('address_line2')
  

    @property
    def address_line3(self):
        return self.attributes.get('address_line3')
  

    @property
    def city(self):
        return self.attributes.get('city')
  

    @property
    def company_number(self):
        return self.attributes.get('company_number')
  

    @property
    def description(self):
        return self.attributes.get('description')
  

    @property
    def directors(self):
        return self.attributes.get('directors')
  

    @property
    def links(self):
        return self.Links(self.attributes.get('links'))
  

    @property
    def name(self):
        return self.attributes.get('name')
  

    @property
    def postal_code(self):
        return self.attributes.get('postal_code')
  


  

  

  

  

  

  

  

  
    class Links(object):
        """Wrapper for the response's 'links' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def creditor(self):
            return self.attributes.get('creditor')
    
  

  

  




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class MandateRequestConstraints(object):
    """A thin wrapper around a mandate_request_constraints, providing easy access to its
    attributes.

    Example:
      mandate_request_constraints = client.mandate_request_constraints.get()
      mandate_request_constraints.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def end_date(self):
        return self.attributes.get('end_date')
  

    @property
    def max_amount_per_payment(self):
        return self.attributes.get('max_amount_per_payment')
  

    @property
    def periodic_limits(self):
        return self.attributes.get('periodic_limits')
  

    @property
    def start_date(self):
        return self.attributes.get('start_date')
  


  

  

  

  




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

import json

import requests
import responses
from nose.tools import (
  assert_equal,
  assert_is_instance,
  assert_is_none,
  assert_is_not_none,
  assert_not_equal,
  assert_raises
)

from gocardless_pro.errors import MalformedResponseError
from gocardless_pro import resources
from gocardless_pro import list_response

from .. import helpers
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class InstitutionsService(base_service.BaseService):
    """Service class that provides access to the institutions
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.Institution
    RESOURCE_NAME = 'institutions'


    def list(self,params=None, headers=None):
        """List Institutions.

        Returns a list of supported institutions.
        
        <p class="deprecated-notice"><strong>Deprecated</strong>: This list
        institutions endpoint
        is no longer supported. We strongly recommend using the
        [List Institutions For Billing
        Request](#institutions-list-institutions-for-billing-request)
        instead.</p>

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of Institution instances
        """
        path = '/institutions'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  

    def list_for_billing_request(self,identity,params=None, headers=None):
        """List institutions for Billing Request.

        Returns all institutions valid for a Billing Request.
        
        This endpoint is currently supported only for FasterPayments.

        Args:
              identity (string): Unique identifier, beginning with "BRQ".
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of Institution instances
        """
        path = self._sub_url_params('/billing_requests/:identity/institutions', {
          
            'identity': identity,
          })
        
        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=False)
        return self._resource_for(response)
  



# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class MandateRequestConstraintsService(base_service.BaseService):
    """Service class that provides access to the mandate_request_constraints
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.MandateRequestConstraints
    RESOURCE_NAME = 'mandate_request_constraints'




# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

from . import base_service
from .. import resources
from ..paginator import Paginator
from .. import errors

class VerificationDetailsService(base_service.BaseService):
    """Service class that provides access to the verification_details
    endpoints of the GoCardless Pro API.
    """

    RESOURCE_CLASS = resources.VerificationDetail
    RESOURCE_NAME = 'verification_details'


    def create(self,params=None, headers=None):
        """Create a verification detail.

        Creates a new verification detail

        Args:
              params (dict, optional): Request body.

        Returns:
              VerificationDetail
        """
        path = '/verification_details'
        
        if params is not None:
            params = {self._envelope_key(): params}

        response = self._perform_request('POST', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)
  

    def list(self,params=None, headers=None):
        """List verification details.

        Returns a list of verification details belonging to a creditor.

        Args:
              params (dict, optional): Query string parameters.

        Returns:
              ListResponse of VerificationDetail instances
        """
        path = '/verification_details'
        

        response = self._perform_request('GET', path, params, headers,
                                         retry_failures=True)
        return self._resource_for(response)

    def all(self, params=None):
        if params is None:
            params = {}
        return Paginator(self, params)
    
  
